@@ -88,6 +88,7 @@ static void     location_entry_start_search    (YelpLocationEntry *entry,
 static void     location_entry_cancel_search   (YelpLocationEntry *entry);
 static void     location_entry_set_entry       (YelpLocationEntry *entry,
                                                 gboolean           emit);
+static gboolean location_entry_start_pulse     (gpointer           data);
 static gboolean location_entry_pulse           (gpointer           data);
 static void     location_entry_set_completion  (YelpLocationEntry *entry,
                                                 GtkTreeModel      *model);
@@ -224,6 +225,10 @@ enum {
     COMPLETION_COL_FLAGS
 };
 
+enum {
+    COMPLETION_FLAG_ACTIVATE_SEARCH = 1<<0
+};
+
 enum {
     LOCATION_SELECTED,
     SEARCH_ACTIVATED,
@@ -807,9 +812,8 @@ location_entry_set_entry (YelpLocationEntry *entry, gboolean emit)
             gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
                                                GTK_ENTRY_ICON_PRIMARY,
                                                "yelp-page-symbolic");
-            if (priv->pulse > 0)
-                g_source_remove (priv->pulse);
-            priv->pulse = g_timeout_add (80, location_entry_pulse, entry);
+            if (priv->pulse == 0)
+                priv->pulse = g_timeout_add (500, location_entry_start_pulse, entry);
         }
         else {
             gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
@@ -858,6 +862,14 @@ location_entry_set_entry (YelpLocationEntry *entry, gboolean emit)
     }
 }
 
+static gboolean
+location_entry_start_pulse (gpointer data)
+{
+    YelpLocationEntryPrivate *priv = GET_PRIV (data);
+    priv->pulse = g_timeout_add (80, location_entry_pulse, data);
+    return FALSE;
+}
+
 static gboolean
 location_entry_pulse (gpointer data)
 {
@@ -882,6 +894,7 @@ location_entry_pulse (gpointer data)
     }
     else {
         gtk_entry_set_progress_fraction (GTK_ENTRY (priv->text_entry), 0.0);
+        priv->pulse = 0;
     }
 
     return flags & LOCATION_ENTRY_IS_LOADING;
@@ -1132,8 +1145,33 @@ cell_set_completion_text_cell (GtkCellLayout     *layout,
                                YelpLocationEntry *entry)
 {
     gchar *title, *desc, *color, *text;
+    gint flags;
+    GList *cells, *cur;
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
+    gtk_tree_model_get (model, iter, COMPLETION_COL_FLAGS, &flags, -1);
+    if (flags & COMPLETION_FLAG_ACTIVATE_SEARCH) {
+        title = g_strdup_printf (_("Search for “%s”"),
+                                 gtk_entry_get_text (GTK_ENTRY (priv->text_entry)));
+        text = g_markup_printf_escaped ("<span size='larger' font_weight='bold'>%s</span>", title);
+        g_object_set (cell, "markup", text, NULL);
+        g_free (text);
+        g_free (title);
+
+        color = yelp_settings_get_color (yelp_settings_get_default (),
+                                         YELP_SETTINGS_COLOR_YELLOW_BASE);
+        cells = gtk_cell_layout_get_cells (layout);
+        for (cur = cells; cur; cur = cur->next) {
+            g_object_set (cur->data,
+                          "cell-background", color,
+                          "cell-background-set", TRUE,
+                          NULL);
+        }
+        g_list_free (cells);
+        g_free (color);
+        return;
+    }
+
     gtk_tree_model_get (model, iter,
                         COMPLETION_COL_TITLE, &title,
                         COMPLETION_COL_DESC, &desc,
@@ -1153,6 +1191,11 @@ cell_set_completion_text_cell (GtkCellLayout     *layout,
     g_object_set (cell, "markup", text, NULL);
     g_free (text);
     g_free (title);
+
+    cells = gtk_cell_layout_get_cells (layout);
+    for (cur = cells; cur; cur = cur->next)
+        g_object_set (cur->data, "cell-background-set", FALSE, NULL);
+    g_list_free (cells);
 }
 
 static gboolean
@@ -1165,6 +1208,7 @@ entry_match_func (GtkEntryCompletion *completion,
     gchar *title, *desc, *titlecase = NULL, *desccase = NULL;
     gboolean ret = FALSE;
     gchar **strs;
+    gint flags;
     GtkTreeModel *model = gtk_entry_completion_get_model (completion);
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
     static GRegex *nonword = NULL;
@@ -1174,9 +1218,13 @@ entry_match_func (GtkEntryCompletion *completion,
     if (nonword == NULL)
         return FALSE;
 
+    gtk_tree_model_get (model, iter, COMPLETION_COL_FLAGS, &flags, -1);
+    if (flags & COMPLETION_FLAG_ACTIVATE_SEARCH)
+        return TRUE;
+
     gtk_tree_model_get (model, iter,
-                        HISTORY_COL_TITLE, &title,
-                        HISTORY_COL_DESC, &desc,
+                        COMPLETION_COL_TITLE, &title,
+                        COMPLETION_COL_DESC, &desc,
                         -1);
     if (title) {
         titlecase = g_utf8_casefold (title, -1);
@@ -1212,8 +1260,16 @@ entry_completion_sort (GtkTreeModel *model,
                        gpointer      user_data)
 {
     gint ret = 0;
+    gint flags1, flags2;
     gchar *key1, *key2;
 
+    gtk_tree_model_get (model, iter1, COMPLETION_COL_FLAGS, &flags1, -1);
+    gtk_tree_model_get (model, iter2, COMPLETION_COL_FLAGS, &flags2, -1);
+    if (flags1 & COMPLETION_FLAG_ACTIVATE_SEARCH)
+        return 1;
+    else if (flags2 & COMPLETION_FLAG_ACTIVATE_SEARCH)
+        return -1;
+
     gtk_tree_model_get (model, iter1, COMPLETION_COL_ICON, &key1, -1);
     gtk_tree_model_get (model, iter2, COMPLETION_COL_ICON, &key2, -1);
     ret = yelp_settings_cmp_icons (key1, key2);
@@ -1245,8 +1301,15 @@ entry_match_selected (GtkEntryCompletion *completion,
 {
     YelpUri *base, *uri;
     gchar *page, *xref;
+    gint flags;
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
+    gtk_tree_model_get (model, iter, COMPLETION_COL_FLAGS, &flags, -1);
+    if (flags & COMPLETION_FLAG_ACTIVATE_SEARCH) {
+        entry_activate_cb (GTK_ENTRY (priv->text_entry), entry);
+        return TRUE;
+    }
+
     g_object_get (priv->view, "yelp-uri", &base, NULL);
     gtk_tree_model_get (model, iter, COMPLETION_COL_PAGE, &page, -1);
 
@@ -1315,9 +1378,9 @@ view_loaded (YelpView          *view,
                                                      NULL, NULL);
             g_hash_table_insert (completions, g_strdup (doc_uri), completion);
             if (document != NULL) {
+                GtkTreeIter iter;
                 ids = yelp_document_list_page_ids (document);
                 for (i = 0; ids[i]; i++) {
-                    GtkTreeIter iter;
                     gchar *title, *desc, *icon;
                     gtk_list_store_insert (GTK_LIST_STORE (base), &iter, 0);
                     title = yelp_document_get_page_title (document, ids[i]);
@@ -1334,6 +1397,11 @@ view_loaded (YelpView          *view,
                     g_free (title);
                 }
                 g_strfreev (ids);
+                gtk_list_store_insert (GTK_LIST_STORE (base), &iter, 0);
+                gtk_list_store_set (base, &iter,
+                                    COMPLETION_COL_ICON, "edit-find-symbolic",
+                                    COMPLETION_COL_FLAGS, COMPLETION_FLAG_ACTIVATE_SEARCH,
+                                    -1);
             }
             g_object_unref (base);
         }
